/* 
 Copyright 2005,2006 Kapor Enterprises, Inc. and Todd Agulnick.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at 
 
 	http://www.apache.org/licenses/LICENSE-2.0
 	
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 
 rdfserializer.js: implements RDF/XML serialization. We had to roll our
 own here because the version supplied by Mozilla outputs dates
 in a format that the parser can't read.
 */

function RDFAccess(datasource) {
	this._init();
}

RDFAccess.prototype =
{
	rdfs: null,
	container: null,
	rdfcu: null,
	resType: null,
	resInstanceOf: null,
	
	
	_init: function()
	{
		// get the RDF Service, for manufacturing resources
		this.rdfs = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(
			Components.interfaces.nsIRDFService);
		

		// get an RDF container so we can process a folder's children
		this.container = Components.classes["@mozilla.org/rdf/container;1"]
			.createInstance(Components.interfaces.nsIRDFContainer);

		// get the container utilities for testing whether a resource is a container
		this.rdfcu = Components.classes["@mozilla.org/rdf/container-utils;1"].
			getService(Components.interfaces.nsIRDFContainerUtils);

		// set up some resources we'll need
		this.resType = this.GetResource("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
		this.resInstanceOf = this.GetResource("http://www.w3.org/1999/02/22-rdf-syntax-ns#instanceOf");
	
	},
	
	GetResource: function(literal)
	{
		return this.rdfs.GetResource(literal);
	}	
} 
function Serializer(datasource) {
	this._init(datasource);
}

Serializer.prototype = {
	namespaces: null,
	ns: null,
	ds: null,
	r: null,
	doc: null,

	_init:  function(datasource) {
		
		this.ds = datasource;
		
		// get our RDF helper object
		this.r = new RDFAccess();
		
		// create a new document into which we'll convert our RDF
//		this.doc = document.implementation.createDocument("","",null);
    this.doc = Components.classesByID["{3a9cd622-264d-11d4-ba06-0060b0fc76dd}"].
      createInstance(Components.interfaces.nsIDOMDOMImplementation).
      createDocument("", "", null);
		
		var rootElement = this.doc.createElement("RDF:RDF");

		// get our namespaces ready
		this.namespaces = new Array("http://www.w3.org/1999/02/22-rdf-syntax-ns",
			"http://home.netscape.com/WEB-rdf",
			"http://home.netscape.com/NC-rdf",
			"http://developer.mozilla.org/rdf/vocabulary/forward-proxy");
			
		this.ns = new Array("RDF", "NS", "NC", "FP");
	
		// Set up the namespaces
		for (var index = 0; index < this.namespaces.length; ++index) {
			rootElement.setAttribute("xmlns:" + this.ns[index], this.namespaces[index] + "#");
		}
		
		this.doc.appendChild(rootElement);
	},
		
		
	FormatDate : function (prdate)
	{
		var d = new Date(prdate / 1000);
		var result = d.toUTCString();
		
		var microseconds = (d.getMilliseconds() * 1000) + (prdate % 1000);
		microseconds = microseconds + "";	// convert to string
		
		microseconds = "000000".substring(0, 6 - microseconds.length) + microseconds;	// format microseconds as 6 digits
		
		return result + " +" + microseconds;
	},
		 
		
	FixName : function(name)
	{
		var parts = name.split("#");
		var pref = "Unknown";
	
		for (var index = 0; index < this.namespaces.length; ++index) {
			if (this.namespaces[index] == parts[0]) {
				pref = this.ns[index];
				break;
			}
		}
	
		
		return pref + ":" + parts[1];
	},
	
	
	BuildContainerElement : function(resource, resources)
	{
		var target = this.ds.GetTarget(resource, this.r.resInstanceOf, true);
		var containerElement;
		var nodes = [];
		
		if (target instanceof Components.interfaces.nsIRDFResource) {
			containerElement = this.doc.createElement(this.FixName(target.Value));
			containerElement.setAttribute("RDF:about", resource.Value);
			
			// get the sequence elements and append 'em as children
			this.r.container.Init(this.ds, resource);
			
			var children = this.r.container.GetElements();
			
			while (children.hasMoreElements()) {
				var child = children.getNext().QueryInterface(Components.interfaces.nsIRDFResource);

				nodes.push(child);
				var childElement = this.doc.createElement("RDF:li");
				childElement.setAttribute("RDF:resource", child.Value);
				containerElement.appendChild(childElement);
			}
		}

    // add children to resources in reverse order
    while (nodes.length > 0) {
      resources.push(nodes.pop());
    }
          
		return containerElement;
	},
						

  AttachAttributes: function(resource, element)
  {
		var predicates = this.ds.ArcLabelsOut(resource);
		
		while (predicates.hasMoreElements()) {
			var predicate = predicates.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
			var pred = this.FixName(predicate.Value);
			var target = this.ds.GetTarget(resource, predicate, true);
			
			// skip these, as they've been processed elsewhere
			if (pred == "RDF:type" || pred == "RDF:nextVal" || pred == "RDF:instanceOf" ||
					this.r.rdfcu.IsOrdinalProperty(predicate)) {
				continue;
			}
			
			// skip these, as we don't care to persist them
			if (pred == "NC:ID" || pred == "NS:LastCharset" || pred == "FP:forward-proxy" || pred == "NC:Icon")
			 continue;
			
			if (target instanceof Components.interfaces.nsIRDFResource) {
					element.setAttribute(pred, target.Value);
			} else if (target instanceof Components.interfaces.nsIRDFLiteral) {
					element.setAttribute(pred, target.Value);
			} else if (target instanceof Components.interfaces.nsIRDFDate) {
					var dateelement = this.doc.createElement(pred);
					dateelement.setAttribute("NC:parseType", "Date");
					var datenode = this.doc.createTextNode(this.FormatDate(target.Value));
					dateelement.appendChild(datenode);
					element.appendChild(dateelement);
					
			} else {
				continue;		// not a type we can handle
			}
		}
	},
    
	Serialize: function()
	{
		// Get the root folder
		var resources = [this.r.GetResource("NC:BookmarksTopRoot")];
		
		while (resources.length > 0) {
			var resource = resources.pop().QueryInterface(Components.interfaces.nsIRDFResource);
	
			// create an element whose type matches the RDF:type of the resource
			var type = this.ds.GetTarget(resource, this.r.resType, true);
			if (type instanceof Components.interfaces.nsIRDFResource) {
				var element = this.doc.createElement(this.FixName(type.Value));
				element.setAttribute("RDF:about", resource.Value);
				
				// if this element is a container, do the special processing required for containers
				// note that this will also push children onto the resources stack for processing
				// (but don't do any of this processing for Livemarks; skip the children
				if (this.r.rdfcu.IsContainer(this.ds, resource) && type.Value != "http://home.netscape.com/NC-rdf#Livemark") {
					this.doc.documentElement.appendChild(this.BuildContainerElement(resource, resources));
				}
				
					// now create remaining elements as attributes of this element
				this.AttachAttributes(resource, element);
				
				this.doc.documentElement.appendChild(element);
			} else {
			
				// we're trying to serialize a resource that (probably) has no assertions
				// most likely, a parent folder was modified and this resource is meant to
				// be a separator. But seps don't work too well. Argh! Fake it for now.
				var element = this.doc.createElement("NC:BookmarkSeparator");
				element.setAttribute("RDF:about", resource.Value);
				this.doc.documentElement.appendChild(element);
			}
		}
		
		// create a serializer
		var xmlserializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"]
				.createInstance(Components.interfaces.nsIDOMSerializer);
		var xml = xmlserializer.serializeToString(this.doc);

    // replace control chars w/ "_" to prevent bustage
		return xml.replace(/[\x01-\x1f]/g, "_");
	}
}
