/* 
 Copyright 2005,2006 Kapor Enterprises, Inc. and Todd Agulnick.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at 
 
 	http://www.apache.org/licenses/LICENSE-2.0
 	
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 
 remote.js: implements network management (gDownloader and gUploader)
 for reading and writing a file via FTP or WebDAV.
 
 */

function alert(msg)
{
  Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
    getService(Components.interfaces.nsIPromptService).
	  alert(null, "Foxmarks Bookmark Synchronizer", msg); 
}

var gDownloader =
{
	channel: null,
	streamLoader: null,
	data: null,
	callback: null,
	
	start: function(uri, etag, callback) {
	
		if(!uri)
			return false;
			
		this.callback = callback;
		
		try {
			var ioService = Components.classes["@mozilla.org/network/io-service;1"]
				.getService(Components.interfaces.nsIIOService);
			this.streamLoader = Components.classes["@mozilla.org/network/stream-loader;1"]
				.createInstance(Components.interfaces.nsIStreamLoader);
			this.channel = ioService.newChannelFromURI(ioService.newURI(uri, "UTF-8", null));
			if (this.channel instanceof Components.interfaces.nsIHttpChannel) {
				this.channel.loadFlags |= Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE;
				SetUserAgent(this.channel);
        if (etag.length > 0) {
					this.channel.setRequestHeader("If-None-Match", etag, false);
				}
			}
			this.streamLoader.init(this.channel, this, null);
		}catch(e) {
			alert(e); 
			return false;
		}
		
		return true;
	},

	cancel: function()
	{
		if (this.channel && this.channel.isPending())
			this.channel.cancel(0x804b0002);
	},

	onStreamComplete : function(loader, ctxt, status, resultLength, result)
	{
		var etag = "";

		if (this.channel instanceof Components.interfaces.nsIHttpChannel) {
			try {
				var response = 200;
				response = this.channel.responseStatus;
				LogWrite("Response is " + response);
//				LogWrite("Download Request Headers:");
//				this.channel.visitRequestHeaders(this);
//				LogWrite("Download Response Headers:");
//				this.channel.visitResponseHeaders(this);
				etag = this.channel.getResponseHeader("Etag");
			} catch (e) { }
			if (response != 200 && response != 201 && response != 204) {
				status = response;
			}
		}
	
		this.data = "";
		
		if(status == 0 || status == 403) {
			if (typeof(result) == "string") {
				this.data = result;
			} else {
				while(result.length > (256*192)) {
					this.data += String.fromCharCode.apply(this, result.splice(0,256*192));
				}
				this.data += String.fromCharCode.apply(this, result);
			}

			var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
					.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
			converter.charset = "UTF-8";
			this.data = converter.ConvertToUnicode(this.data);
		} else {
			if (status == 0x804b0016) {		// map FTP CWD error to 404 to make life easier for caller
				status = 404;
			}
		}
		
		if(this.callback)
			this.callback(status, etag);
		  
	},

	onDataAvailable: function(channel, context, input, sourceOffset, count)
	{
		dump("gDownloader: in onDataAvailable\n");
	},
	
	onStartRequest: function(channel, context)
	{
	},
	
	onStopRequest: function(channel, context, status)
	{
	},
	
	visitHeader: function (header, value)
	{
		LogWrite("Download Header: " + header + " : " + value);
	}
	

}

var gUploader = {
	channel : null,
	dataCopy : null,
	recvdata : "",
	stream: null,
	callback: null,

	start: function(uri, etag, string, callback)
	{
		if (!uri || !string)
			return false;
			
		this.callback = callback;
		this.dataCopy = string;
		this.recvdata = "";

		// First, get and initialize the converter
		var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
			.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
		converter.charset = "UTF-8";
		
		// Now, convert a string to an nsIInputStream
		this.stream = converter.convertToInputStream(this.dataCopy);
		
		LogWrite("Writing " + this.stream.available() + " bytes.");
			
		const ioService = Components.classes["@mozilla.org/network/io-service;1"].
			getService(Components.interfaces.nsIIOService);
			
		this.channel = ioService.newChannelFromURI(ioService.newURI(uri, "UTF-8", null));
		this.channel.QueryInterface(Components.interfaces.nsIUploadChannel);
			
		try {
		  if (this.channel instanceof Components.interfaces.nsIHttpChannel) {
		    SetUserAgent(this.channel);
   			if (etag && etag.length > 0 && !gSettings.disableIfMatchOnPut) {
  					this.channel.setRequestHeader("If-Match", etag, false);
  			}
			}
			this.channel.setUploadStream(this.stream, "text/xml; charset=UTF-8", -1);
			this.channel.asyncOpen(this, null);
			return true;
		} catch (e) {
			alert(e);
			return false
		}
	},
	
	onDataAvailable: function(channel, context, input, sourceOffset, count)
	{
		var instream = Components.classes["@mozilla.org/scriptableinputstream;1"]
			.createInstance(Components.interfaces.nsIScriptableInputStream);
		
		instream.init(input);
		this.recvdata += instream.read(count);
		LogWrite("onDataAvailable: >>>\n" + this.recvdata + "\n<<<");
	
	},
	
	onStartRequest: function(channel, context)
	{
	},
	
	onStopRequest: function(channel, context, status)
	{
		// FTP transfers return an error code directly by status
		// HTTP & HTTPS (WebDAV) transfers return network status via status
		// and an HTTP response code via responseStatus
		// If the response status is one of the few acceptable ones
		// (200: OK, 201: Created, 204: No Content), we're cool.
		// Otherwise pass the response code (which presumably indicates failure)
		// through as an error
		
		var etag = "";
		
		if (channel instanceof Components.interfaces.nsIHttpChannel) {
			try {
				var response = 200;
				response = channel.responseStatus;
//				LogWrite("Upload Request Headers:");
//				channel.visitRequestHeaders(this);
//				LogWrite("Upload Response Headers:");
//				channel.visitResponseHeaders(this);
				etag = channel.getResponseHeader("Etag");
			} catch (e) {}
			if (response != 200 && response != 201 && response != 204) {
				status = response;
			}
		}
		
		if (this.callback) {
			this.callback(status, etag);
		}
	},
	
	cancel: function()
	{
		if (this.channel && this.channel.isPending())
			this.channel.cancel(0x804b0002);
	},
	
	visitHeader: function (header, value)
	{
		LogWrite("Upload Header: " + header + " : " + value);
	}

}

function SetUserAgent(channel)
{
  channel.setRequestHeader("User-Agent", 
    channel.getRequestHeader("User-Agent") + 
      " Foxmarks/" + FoxmarksVersion(), false);
}

function MapError(status)
{
	var error = "";
	
	status = status & 0x0000ffff;

  try {
     error = Bundle().GetStringFromName("error." + status);
  } catch (e) {
     error = Bundle().formatStringFromName("error.unknown", [status], 1);
  }
  
	return error;
} 
