/* 
 Copyright 2005,2006 Kapor Enterprises, Inc. and Todd Agulnick.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at 
 
 	http://www.apache.org/licenses/LICENSE-2.0
 	
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 
 foxmarks-settings.js: implements FoxmarksSettings, an object that wraps
 access to persistent settings, both user settings and internal stored values.
   
 */

function Bundle()
{
  var sb = Components.classes["@mozilla.org/intl/stringbundle;1"]
      .getService(Components.interfaces.nsIStringBundleService)
      .createBundle("chrome://foxmarks/locale/foxmarks.properties");
  return sb;
}

function formatDate(d)
{
 /* args to be passed in:
 
 1: Abbreviated month name
 2: Month Number
 3: Day of month
 4: Hours (12 hour clock)
 5: Hours (24 hour clock)
 6: Minutes
 7: am/pm string
 
 Hours24 and minutes are always 2 digits.
 
 So, to get our default en-US time string, we use:
 
 1$S %2$S, %4$S:%6$S %7$S%3$.0S%5$.0S
 
 which gets us something like:
 
 Jan 3, 12:09 pm
 
 */
 
	var rv = "";
	var sb = Bundle();

  var mon = d.getMonth() + 1;
  var day = d.getDate();
	var hours24 = d.getHours();
	var hours;
	var minutes = d.getMinutes();
	
	var ampm = sb.GetStringFromName("date." + (hours24 < 12 ? "am" : "pm"));
	
	if (hours24 > 12)
		hours = hours24 - 12;
  else if (hours24 == 0)
		hours = 12;
  else
    hours = hours24;
    
  if (minutes < 10) minutes = "0" + minutes;
  if (hours24 < 10) hours24 = "0" + hours24;

	var args = [sb.GetStringFromName("date.mon." + mon), String(mon),
      String(day), String(hours), String(hours24), String(minutes), ampm];
	
  rv = sb.formatStringFromName("date.format", args, args.length);
  
	return rv;
}

/**
* Convert a string containing binary values to hex.
* Shamelessly stolen from nsUpdateService.js
*/
function binaryToHex(input) {
 var result = "";
 for (var i = 0; i < input.length; ++i) {
   var hex = input.charCodeAt(i).toString(16);
   if (hex.length == 1)
     hex = "0" + hex;
   result += hex;
 }
 return result;
}

function hex_md5(string)
{
	var arr = new Array();
	
	for (var i = 0; i < string.length; ++i)
		arr[i] = string.charCodeAt(i);
		
	try {
  	var hash = Components.classes["@mozilla.org/security/hash;1"].
    			createInstance(Components.interfaces.nsICryptoHash);
		hash.initWithString("md5");
		hash.update(arr, arr.length);
    var digest = binaryToHex(hash.finish(false));
	} catch (e) {
		alert(e);
	} 
	return digest.slice(16);	// 8 bytes seems sufficient for our purposes
}

// Return the version string associated with the currently installed version
function FoxmarksExtensionManagerLiteral(value)
{
  var rdfs = Components.classes["@mozilla.org/rdf/rdf-service;1"].
    getService(Components.interfaces.nsIRDFService);
  var ds = Components.classes["@mozilla.org/extensions/manager;1"].
    getService(Components.interfaces.nsIExtensionManager).datasource;
  var s = rdfs.GetResource("urn:mozilla:item:foxmarks@kei.com");
  var p = rdfs.GetResource("http://www.mozilla.org/2004/em-rdf#" + value);
  var t = ds.GetTarget(s, p, true);
  if (t instanceof Components.interfaces.nsIRDFLiteral)
    return t.Value;
  else
    return "unknown";
}

function FoxmarksVersion()
{
  return FoxmarksExtensionManagerLiteral("version");
}

function FoxmarksUpdateAvailable()
{
  return FoxmarksExtensionManagerLiteral("availableUpdateURL") != "none";
}

function FoxmarksSettings()
{
	this._init();
}

FoxmarksSettings.prototype = {
	prefs: null,
	syncStart: null,
	
	_init: function()
	{
	
		var prefsService = Components.classes["@mozilla.org/preferences-service;1"].
			getService(Components.interfaces.nsIPrefService);
			
		this.prefs = prefsService.getBranch("foxmarks.");
	},

	getCharPref: function(string, def)
	{
		var result;
		
		try {
			result = this.prefs.getCharPref(string);
		} catch (e) {
			result = def;
		}
		
		return result;
	},

	getIntPref: function(string, def)
	{
		var result;
		
		try {
			result = this.prefs.getIntPref(string);
		} catch (e) {
			result = def;
		}
		
		return result;
	},


	getBoolPref: function(string, def)
	{
		var result;
		
		try {
			result = this.prefs.getBoolPref(string);
		} catch (e) {
			result = def;
		}
		
		return result;
	},


	// get fundamental settings

	get serverType() {
		return this.getCharPref("serverType", "http");
	},
	
	get hostname() {
		return this.getCharPref("hostname", this.wizardHostName);
	},
	
	get wizardHostName() {
	 return this.getCharPref("wizardHostName", "sync.foxcloud.com");
	},
	
	get username() {
		return this.getCharPref("username", "");
	},
	
	get password(){
		var host= new Object();
		var user= new Object();
		var pass= new Object();
		try{
			var pmInternal = Components.classes["@mozilla.org/passwordmanager;1"]
				.createInstance(Components.interfaces.nsIPasswordManagerInternal);
			pmInternal.findPasswordEntry(this.hostname,this.username,"",host,user,pass);
			return pass.value;
		} catch(e) {
			return "";
		}
	},
	
	get path() {
		return this.getCharPref("path", "/home/{username}/");
	},
	
	get hash() {
		
		if (this.lastSynchURL.length > 0) {						// perform upgrade
			var oldURL = this.lastSynchURL;
			this.prefs.clearUserPref("lastSynchURL");		// delete this; no longer used
			this.prefs.setCharPref(hex_md5(oldURL) + "." + "etag",
				this.getCharPref("etag", ""));
			this.prefs.setCharPref(hex_md5(oldURL) + "." + "lastSynchDate",
				this.getCharPref("lastSynchDate", ""));	// set lastSynchDate in hashed sub-branch
			try {
				this.prefs.clearUserPref("lastSynchDate");
				this.prefs.clearUserPref("etag");
			} catch (e) {}
		}
		
		return hex_md5(this.URL) + ".";
	},
			
				
	get lastSynchURL() {	// deprecated
		return this.getCharPref("lastSynchURL", "");
	},
	
	get lastSynchDate() {
		return this.getCharPref(this.hash + "lastSynchDate", "");
	},
	
  get minutesSinceLastSync() {
    if (!this.HaveSynched)
      return 0;
      
    var syncMS = new Date(this.lastSynchDate).getTime();
    var nowMS = new Date().getTime();
    return (nowMS - syncMS) / 60000;
  
  },
  
  get daysSinceLastUpdateNag() {
    if (!this.lastNagDate)
      return Infinity;
    var updateMS = new Date(this.lastNagDate).getTime();
    var nowMS = new Date().getTime();
    return (nowMS - updateMS) / (60000 * 60 * 24);
  },
  
  get lastNagDate() {
    return this.getCharPref("lastNagDate", null);
  },
  
  set lastNagDate(string) {
    this.prefs.setCharPref("lastNagDate", string);
  },
	
	get lastSynchDisplayDate() {
		if (!this.HaveSynched) {
      return Bundle().GetStringFromName("msg.neversynced");
		} else {
			var d = new Date(this.lastSynchDate);
			return formatDate(d);
		}
	},

	get etag() {
		return this.getCharPref(this.hash + "etag", "");
	},
	
	get token() {
    return this.getCharPref(this.hash + "token", "");
  },
	
	get synchOnTimer() {
		return this.getBoolPref("synchOnTimer", true);
	},
	
	get autoSynchFreq() {
		return this.getIntPref("autoSynchFreq", 60);
	},
	
	get debugUI() {
	  return this.getBoolPref("debugUI", false);
	},
	
	get wizardRan() {
	 return this.getBoolPref("wizardRan", false);
	},
	
	set wizardRan(bool) {
		this.prefs.setBoolPref("wizardRan", bool);
	},
	
	get wizardSuppress() {
     return this.getBoolPref("wizardSuppress", false);
  },
  
  set wizardSuppress(bool) {
    this.prefs.setBoolPref("wizardSuppress", bool);
  },


 	get disableIfMatchOnPut() {
 		return this.getBoolPref("disableIfMatchOnPut", false);
 	},


	// set fundamental settings
	resetServerSettings: function() {
	 try { this.prefs.clearUserPref("serverType"); } catch (e) {}
	 try { this.prefs.clearUserPref("hostname"); } catch (e) {}
	 try { this.prefs.clearUserPref("path"); } catch (e) {}
	},

  resetAll: function () {
    var list = this.prefs.getChildList("", {});
    for (var i = 0; i < list.length; ++i) {
      try { this.prefs.clearUserPref(list[i]); } catch (e) {}
    }
  },

	set serverType(string) {
		this.prefs.setCharPref("serverType", string);
	},
	
	set hostname(string) {
		this.prefs.setCharPref("hostname", string);
	},
	
	set username(string) {
		this.prefs.setCharPref("username", string);
	},
	
	set password(aStr){
		var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
			.createInstance(Components.interfaces.nsIPasswordManager);
		try{ 
			passwordManager.removeUser(this.hostname, this.username);
		} catch(e) {}
			passwordManager.addUser(this.hostname, this.username, aStr);
	},
	
	set path(string)
	{
		if (string.substr(-1, 1) != "/")
			string = string + "/";
			
		if (string.substr(0,1) != "/")
			string = "/" + string;
			
		this.prefs.setCharPref("path", string);
	},
	
	set lastSynchDate(string) {
		this.prefs.setCharPref(this.hash + "lastSynchDate", string);
	},

	set etag(string) {
		this.prefs.setCharPref(this.hash + "etag", string);
	},
	
	set token(string) {
    this.prefs.setCharPref(this.hash + "token", string);
    LogWrite("Token set to " + string);
  },
		
	set autoSynchFreq(integer) {
		this.prefs.setIntPref("autoSynchFreq", integer);
	},
	
	set synchOnTimer(bool) {
		this.prefs.setBoolPref("synchOnTimer", bool);
	},
	
	set debugUI(bool) {
  	 this.prefs.setBoolPref("debugUI", bool);
	},
	
	// get calculated settings
	get calcPath()
	{
	   return this.path.replace("{username}", this.username);
	},
	   
	get downloadURL()
	{
		var pw = this.password;
		var user = this.username;

		if (pw.length > 0)
			user += ":" + pw;
			
		if (user.length > 0)
			user += "@";
			
		return this.serverType + "://" + user + this.hostname + this.calcPath + "foxmarks.xml";
	},
	
	get URL()
	{
		var user = this.username;
		
		if (user.length > 0)
			user += "@";
			
		return this.serverType + "://" + user + this.hostname + this.calcPath + "foxmarks.xml";
	},
	
	
	get HaveSynched()
	{
		return (this.lastSynchDate.length > 0);
	},


  get NowAsGMT()
  {
    var d = new Date();
    return d.toGMTString();
  },
    
  get SyncStarted()
  {
    return this.syncStart;
  },
    
  SyncStart: function()
  {
    this.syncStart = this.NowAsGMT;
    return this.syncStart;
  },

	SyncComplete: function(etag)
	{
   this.lastSynchDate = this.NowAsGMT;
   
   // clear the syncStart date for the next run
   this.syncStart = null;
   
		if (etag.length > 0)
			this.etag = etag;
	}
}

var gSettings = new FoxmarksSettings();
