/* 
 Copyright 2005,2006 Kapor Enterprises, Inc. and Todd Agulnick.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at 
 
 	http://www.apache.org/licenses/LICENSE-2.0
 	
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 
 foxmarks-setup.js: implements behavior for the Foxmarks Setup Wizard.
   
 */

var gReadyToSyncPrev = null;

function AlertMsg(msgname)
{
  alert(Bundle().GetStringFromName(msgname));
}

function SetupOnLoad()
{
	// if we're re-running the wizard for some reason, get
	// the current settings
	document.getElementById("username").value = gSettings.username;
	document.getElementById("password").value = gSettings.password;
	document.getElementById("password2").value = gSettings.password;
	document.getElementById("existingUsername").value = gSettings.username;
	document.getElementById("existingPassword").value = gSettings.password;
	
	gSettings.wizardRan = true;    // auto launch only once
}

function SetupHasAccount()
{
  if (document.getElementById("hasAccount").selectedItem.value == "yes") {
    document.documentElement.goTo("useExistingAccount");
    return false;
  }
  
  return true;
}

function IsValid(s, min, max, regex)
{
  if (s.length < min || s.length > max)
    return false;
    
  return regex.test(s);
}
    
function IsValidUsername(u) { return IsValid(u, 3, 32, /^[a-zA-Z0-9'._-]+$/); }
function IsValidPassword(p) { return IsValid(p, 5, 16, /^[a-zA-Z0-9'-]+$/); }
function IsValidFirstName(f){ return IsValid(f, 1, 128, /^[^\x01-\x1f]+$/); }
function IsValidLastName(l) { return IsValid(l, 1, 128, /^[^\x01-\x1f]+$/); }
function IsValidEmail(e) { return IsValid(e, 1, 128, /^[a-zA-Z0-9._-]+@([a-zA-Z0-9.-]+\.)+[a-zA-Z0-9.-]{2,4}$/); }
 
function SetupValidateExisting()
{
  if (!IsValidUsername(document.getElementById("existingUsername").value)) {
    AlertMsg("msg.invalidusername");
    document.getElementById("existingUsername").focus();
    return false;
  }
  
  if (!IsValidPassword(document.getElementById("existingPassword").value)) {
    AlertMsg("msg.invalidpassword");
    document.getElementById("existingPassword").focus();
    return false;
  }
  
  var retval = {};
  var arg = {username: document.getElementById("existingUsername").value,
             password: document.getElementById("existingPassword").value};

  PerformAction("verifyAccount", retval, arg);

  if (retval.status != 0) {
    AlertMsg("msg.verifyfailed");
    return false;
  }
  
  return true;
}

function SetNextBasedOnAgree()
{
  document.getElementById("foxmarks-setup").canAdvance =
    document.getElementById("agree").checked;
}

function SetupNewAccount()
{
  if (!IsValidFirstName(document.getElementById("firstname").value)) {
    AlertMsg("msg.invalidfirstname");
    document.getElementById("firstname").focus();
    return false;
  }
  
  if (!IsValidLastName(document.getElementById("lastname").value)) {
    AlertMsg("msg.invalidlastname");
    document.getElementById("lastname").focus();
    return false;
  }
  
  if (!IsValidEmail(document.getElementById("email").value)) {
    AlertMsg("msg.invalidemail");
    document.getElementById("email").focus();
    return false;
  }
  
  if (!IsValidUsername(document.getElementById("username").value)) {
    AlertMsg("msg.invalidusername");
    document.getElementById("username").focus();
    return false;
  }
  
  if (!IsValidPassword(document.getElementById("password").value)) {
    AlertMsg("msg.invalidpassword");
    document.getElementById("password").focus();
    return false;
  }
  
  if (document.getElementById("password").value != document.getElementById("password2").value) {
    AlertMsg("msg.passwordsdontmatch");
    document.getElementById("password").focus();
    return false;
  }
  
  var retval = {};
  var arg = {
    firstname: document.getElementById("firstname").value,
    lastname: document.getElementById("lastname").value,
    email: document.getElementById("email").value,
    username: document.getElementById("username").value,
    password: document.getElementById("password").value };

  PerformAction("createAccount", retval, arg);

  if (retval.status == 0)
    return true;
    
  if (retval.status == 431) {
    AlertMsg("msg.usernameinuse");
    document.getElementById("username").focus();
    return false;
  }

  if (retval.status == 432) {
    AlertMsg("msg.emailinuse");
    document.getElementById("email").focus();
    return false;
  }
    
    
  if (retval.status == 409) {
    AlertMsg("msg.inuse");
    document.getElementById("username").focus();
    return false;
  }
  
  if (retval.status != 0) {
    alert(Bundle().formatStringFromName("msg.accountfailed", [retval.msg], 1));
    return false;
  }
}

var gRemoteFileExists = false;

function SetupCheckFileExists()
{
  if (gSettings.HaveSynched) {    // user is re-running Wizard for some reason...
    gReadyToSyncPrev = document.documentElement.currentPage.pageid;
    document.documentElement.goTo("readyToSync");
    return false;
  }
  
  var retval = {};

  PerformAction("verifyFile", retval, null);
      
  if (!retval.status) {
    document.documentElement.goTo("initialSync");
    gRemoteFileExists = true;
  } else if (retval.status == 404) {
    gReadyToSyncPrev = document.documentElement.currentPage.pageid;
    document.documentElement.goTo("readyToSync");
    gRemoteFileExists = false;
  } else if (retval.status) {
    alert(Bundle().formatStringFromName("msg.accountaccess", [retval.msg], 1));
  }
  return false;
}

function ShowSyncPage()
{
  var op;
  var a = document.getElementById("localOrRemote").selectedItem.value;
  var b = document.getElementById("mergeStart").selectedItem.value;
  
  if (gSettings.HaveSynched) {
    op = "msg.standardsync";
  } else {
    if (!gRemoteFileExists) {
      op = "msg.upload";
    } else {
      if (a == "local") {
        op = "msg.upload";
      } else if (a == "remote") {
        op = "msg.download";
      } else {
        if (b == "local") {
          op = "msg.mergelocal";
        } else {
          op = "msg.mergeremote";
        }
      }
    }
  }
  document.getElementById("operation").value = Bundle().GetStringFromName(op);
}

function SetupPerformSync()
{
  var retval = {}
  var args = {};

  var a = document.getElementById("localOrRemote").selectedItem.value;
  var b = document.getElementById("mergeStart").selectedItem.value;
  
  args.merge = (a == "merge");
  args.remoteIsMaster = args.merge ? (b == "remote") : (a == "remote");
  
  PerformAction("initialSync", retval, args);
      
  if (!retval.status) {
    return true;
  } else {
    alert(Bundle().formatStringFromName("msg.syncfailed", [retval.msg], 1));
  }
  
  return false;
}

function PerformAction(action, retval, args)
{
	var win = window.openDialog("chrome://foxmarks/content/foxmarks-progress.xul",
      "_blank", "chrome,dialog,modal,centerscreen", action, retval, args);
      
  if (retval.helpurl) {
  	openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", url);
  	retval.status = -1;
  	retval.msg = Bundle().GetStringFromname(msg.cancelled);
  }
  
  return;
}

function InitialSyncNext()
{
  if (document.getElementById("localOrRemote").selectedItem.value == "merge") {
    return true;
  } else {
    gReadyToSyncPrev = document.documentElement.currentPage.pageid;
    document.documentElement.goTo("readyToSync");
  }
  return false;
}

function InitialSyncBack()
{
  var page = null;
  
  if (document.getElementById("hasAccount").selectedItem.value == "yes")
    page = "accountVerified";
  else
    page = "accountCreated";
    
  if (page) {
    document.documentElement.goTo(page);
    return false;
  }
  
  return true;
}

function SetupSyncBack()
{
  if (gReadyToSyncPrev) {
    document.documentElement.goTo(gReadyToSyncPrev);
    gReadyToSyncPrev = null;
    return false;
  }
  
  return true;
}

function onWizardCancel()
{
  var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
      .getService(Components.interfaces.nsIPromptService);

  var checkResult = {};
  checkResult.value = gSettings.wizardSuppress;
  var sb = Bundle();

  // Show a confirmation dialog with a check box. Arguments in order are, the parent
  // window, the dialog title, the dialog message, the check box label and
  // a result object for the check box state.
  var ret = ps.confirmCheck(window, sb.GetStringFromName("title.cancelsetup"),
    sb.GetStringFromName("msg.cancelsetup"),
    sb.GetStringFromName("msg.nowizard"),
    checkResult);

  gSettings.wizardSuppress = checkResult.value;
  
  return ret;
}
  
function onWizardFinish()
{
  gSettings.wizardSuppress = true;  // don't need to auto launch again
  return true;
}
