/*
 Copyright 2005,2006 Kapor Enterprises, Inc. and Todd Agulnick.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 foxmarks-service.js: component that implements the "service" interface to
 the core synchronization code.

 */

function LoadJavascript(filename, identifiertype)
{
  if (identifiertype != "undefined") {
    return;   // already defined
  }

  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
    getService(Components.interfaces.mozIJSSubScriptLoader).
    loadSubScript(filename, null);
}


var em;

function ForceUpdate()
{
  // find a window to use as a parent for open
  var topwin = Components.classes['@mozilla.org/appshell/window-mediator;1'].
      getService(Components.interfaces.nsIWindowMediator).
      getMostRecentWindow(null);

  // first, open the extension manager window
  em = topwin.open("chrome://mozapps/content/extensions/extensions.xul?type=extensions",
    "_blank", "chrome,dialog=no,resizable");
  em.addEventListener("pageshow", ExtensionManagerShow, false);
  
  function ExtensionManagerShow(arg) {
    const id = "urn:mozilla:item:foxmarks@kei.com";

    // find our entry in the extension manager's view
    with (em) {
      var children = gExtensionsView.children;

      while (children.length > 0) {
        var item = children.shift()

        if (item.id == id) {       // found it, install it
          gExtensionsView.selectedItem = item;
          var updateitem = gExtensionManager.getItemForID("foxmarks@kei.com");
          var listener = new FoxmarksUpdateCheckListener();
          gExtensionManager.update([updateitem], 1, false, listener);
          break;
        }
      }
    }
  }
}

function FoxmarksUpdateCheckListener() {
}

FoxmarksUpdateCheckListener.prototype = {
  onUpdateStarted: function() {
    LogWrite("onUpdateStarted");
  },

  onUpdateEnded: function() {
    LogWrite("onUpdatedEnded");
  },

  onAddonUpdateStarted: function(addon) {
    LogWrite("onAddonUpdateStarted");
  },

  onAddonUpdateEnded: function(addon, status) {
    LogWrite("onAddonUpdatedEnded, status = " + status);

    const nsIAUCL = Components.interfaces.nsIAddonUpdateCheckListener;
    switch (status) {
    case nsIAUCL.STATUS_UPDATE:
      em.gExtensionManager.addDownloads([addon], 1, true);
      break;
    case nsIAUCL.STATUS_FAILURE:
      break;
    case nsIAUCL.STATUS_DISABLED:
      break;
    }
  },

  /**
   * See nsISupports.idl
   */
  QueryInterface: function(iid) {
    if (!iid.equals(Components.interfaces.nsIAddonUpdateCheckListener) &&
        !iid.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

function UpdateNag()
{
  var sb = Bundle().GetStringFromName;
  
  if (Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
      getService(Components.interfaces.nsIPromptService).
      confirm(null, sb("appname.long"), sb("msg.upgradeAvailable"))) {
    ForceUpdate();
  }
}


function OpenFoxmarksSettingsDialog()
{
   var wm = Components.classes['@mozilla.org/appshell/window-mediator;1'].
     getService(Components.interfaces.nsIWindowMediator);
   var win = wm.getMostRecentWindow("foxmarks:settings");

   if (win) {
     // if it's already open, try to focus it, but no worries
     // if that fails because we're in a modal dependency chain
     // (i.e., user invoked a modal sync from settings dialog)
     try {
       win.focus();
     } catch (e) {}
   } else {
     win = wm.getMostRecentWindow(null);
     win.open("chrome://foxmarks/content/foxmarks-dialog.xul", "_blank", "chrome");
   }
}


// Returns child of node named "name," null if not found

function ChildNode(node, name)
{
  for (var i = 0; i < node.childNodes.length; ++i) {
    var child = node.childNodes.item(i);
    
    if (child.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE &&
        child.localName == name) {
      return child;
    }
  }
  return null;
}

function Handle403()
{
  var sb = Bundle().GetStringFromName;
  var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"].
    createInstance(Components.interfaces.nsIDOMParser);
  var doc = parser.parseFromString(gDownloader.data, "text/xml");
  if (doc.documentElement.localName == "error") {
    var versionNotSupported = ChildNode(doc.documentElement, "version-not-supported");
    if (versionNotSupported) {
      LogWrite("Upgrade required.");
      
      var infoURL = ChildNode(versionNotSupported, "infoURL");
      
      if (!infoURL) {

        if (Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
          getService(Components.interfaces.nsIPromptService).
          confirm(null, sb("appname.long"), sb("msg.upgraderequired"))) {
            ForceUpdate();
        }
      } else {
        if (Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
            getService(Components.interfaces.nsIPromptService).
            confirm(null, sb("appname.long"), sb("msg.emergencyupgrade"))) {
          Components.classes['@mozilla.org/appshell/window-mediator;1'].
            getService(Components.interfaces.nsIWindowMediator).
            getMostRecentWindow("navigator:browser").
            openDialog("chrome://browser/content/browser.xul", "_blank",
              "chrome,all,dialog=no", infoURL.firstChild.nodeValue);
        }
      }
    }
  }
}

function LoadFiles()
{
  LoadJavascript("chrome://foxmarks/content/foxmarks-log.js", typeof(LogWrite));
  LoadJavascript("chrome://foxmarks/content/foxmarks-remote.js", typeof(gDownloader));
  LoadJavascript("chrome://foxmarks/content/foxmarks-settings.js", typeof(gSettings));
  LoadJavascript("chrome://foxmarks/content/foxmarks-bookmarkset.js", typeof(BookmarkSet));
  LoadJavascript("chrome://foxmarks/content/foxmarks-sync.js", typeof(Sync));
  LoadJavascript("chrome://foxmarks/content/foxmarks-merge.js", typeof(Merge));
  LoadJavascript("chrome://foxmarks/content/foxmarks-rdfserializer.js", typeof(Serializer));
}

var logStream = null;

function logFileOpen()
{
  var file = Components.classes['@mozilla.org/file/directory_service;1']
    .getService(Components.interfaces.nsIProperties)
    .get('ProfD', Components.interfaces.nsIFile);

  file.append("foxmarks.log");

  logStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
    .createInstance(Components.interfaces.nsIFileOutputStream);

  // use write, append, create
  logStream.init(file, 0x02 | 0x08 | 0x10, 0664, 0);
}

function logFileClose()
{
  logStream.close();
  logStream = null;
}

var gMerge = {};
var gObserver = null;   // if non-null, we're busy
var gAccount = {};    // used during account-creation process
var gPrompt = false;
var gRemoteIsMaster = false;
var gDoMerge = false;
var gFailureCount = 0;      // number of consecutive times we've failed
var gBackoffUntil = 0;      // if we've been failing, our backoff time (ms since 1970)

function SetProgressMessage(msgname)
{
  if (gObserver) {
    var msg = Bundle().GetStringFromName(msgname);
    gObserver.onFoxmarksProgress(0, msg, false);
  }
}

function ReturnErrorMsg(code, msg)
{
  if (gObserver) {
    try {
        gObserver.onFoxmarksProgress(code, msg, true);
    } catch (e) {}
  }
  gObserver = null;
  LogWrite("Returned error: " + msg + "(" + code + ")");
  gFailureCount++;
  var d = new Date();
  gBackoffUntil = d.getTime() + Math.pow(2, gFailureCount) * 1000 * 60;
}

function ReturnErrorCode(code)
{
  ReturnErrorMsg(code, MapError(code));
}

function ReturnSuccess(msgname)
{
  if (gObserver) {
    try {
      gObserver.onFoxmarksProgress(0, Bundle().GetStringFromName(msgname), true);
    } catch (e) {}
  }
  gObserver = null;
  gFailureCount = 0;
  LogWrite("Success: " + Bundle().GetStringFromName(msgname));
}

function IsBusy(observer)
{
  if (gObserver) {
    try {
      observer.onFoxmarksProgress(-1, Bundle().GetStringFromName("msg.busy"), true);
    } catch (e) {}
    return true;
  } else {
    gObserver = observer;
    return false;
  }
}

function ParseURL(url)
{
  // parse as type :// server / path
  var s = url.indexOf("://");
  if (s < 0) {
    return -1;
  }

  var t = url.indexOf("/", s + 3);
  if (t < 0) {
    return -1;
  }

  gSettings.serverType = url.slice(0, s);
  gSettings.hostname = url.slice(s + 3, t);
  gSettings.path = url.slice(t);

  return 0;

}

function StartUpload(source, etag, token)
{
  SetProgressMessage("progress.writing");
  
  if (!token || !token.length) { // generate a new token
    var d = new Date();
    token = d.getTime().toString(16);
    LogWrite("Generated new token " + token);
  }

  gUploader.start(gSettings.downloadURL, etag, source.SerializeToString(token),
    UploadCallback);
    
  return;
  
  function UploadCallback(status, etag) {
    if (!status) {
      gSettings.SyncComplete(etag);
      gSettings.token = token;
      ReturnSuccess("msg.uploadcompleted");
    } else {
      ReturnErrorCode(status);
    }
  }
}

// Internal callbacks

function VerifyFileCallback(status, etag)
{
  if (!status) {
    var remote = new BookmarkSet();
    if (remote.CreateFromString(gDownloader.data, null) < 0) {
      ReturnErrorCode(1000);
    } else {
      ReturnSuccess("msg.fileexists");
    }
  } else if (status == 404) {
    ReturnErrorMsg(status, Bundle().GetStringFromName("msg.nosyncfile"));
  } else {
   ReturnErrorCode(status);
  }
}

function CreateAccountCallback(status, etag)
{
  LogWrite("CreateAccountCallback: Status is " + status);

  if (!status) {
    try {
      var contentLocation = gUploader.channel.getResponseHeader("Content-Location");
    } catch(e) { ReturnErrorCode(1001); return; }

    if (ParseURL(contentLocation) < 0) {
      ReturnErrorCode(1002);
      return;
    }

    gSettings.username = gAccount.username;
    gSettings.password = gAccount.password;

    LogWrite("CreateAccountCallback parsed as " + gSettings.serverType + ", " +
      gSettings.hostname + ", " + gSettings.path);

    ReturnSuccess("msg.accountcreated");

  } else {
    ReturnErrorCode(status);
  }
}

function DownloadCallbackVerify(status, etag)
{
  if (status) {
    ReturnErrorCode(status);
    return;
  }

//  var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"].
//    createInstance(Components.interfaces.nsIDOMParser);
//
//  var response = parser.parseFromString(gDownloader.data, "text/xml");
//
//  var homedirurl = response.getElementsByTagName("homedirUrl").item(0).firstChild.nodeValue;
//  if (ParseURL(homedirurl) < 0) {
//    ReturnErrorCode(1003);
//    return;
//  }

  gSettings.username = gAccount.username;
  gSettings.password = gAccount.password;

  // until we can derive server settings from Foxcloud API,
  // just reset 'em and assume that defaults produce correct results
  gSettings.resetServerSettings();

  ReturnSuccess("msg.accountverified");
 }

function DownloadCallbackOverwrite(status, etag)
{
  if (!status) {
    CopyRemoteToLocal(false, etag);
  } else {
    ReturnErrorCode(status);  // error
  }
}

function FirstSync(status, etag)
{
  LogWrite("Initial synch");

  if (!status) {
    // We've never synched, but we got a valid file
    switch (LocalOrRemote()) {
      case 0: // local
        LogWrite("User chooses local");
        CopyLocalToRemote(gMerge.value, false, null, true);
        break;

      case 1: // remote
        LogWrite("User chooses remote");
        CopyRemoteToLocal(gMerge.value, etag);
        break;

      case 2: // cancel
        LogWrite("User chooses cancel");
        ReturnSuccess("msg.cancelled");
        break;
    }

  } else if (status == 404) {
    // we've never synched and there's no file on remote server
    LogWrite("File not found; CopyLocalToRemote");
    CopyLocalToRemote(false, false, null, true);
  } else {
    ReturnErrorCode(status);  // error
  }
}

function NormalSync(status, etag)
{
  // we have synched previously
  var dirty = {local: false, remote: false};
  var sb = Bundle().GetStringFromName;
  var attrs = {};

  switch (status) {
    case 0: // normal case
      SetProgressMessage("progress.syncing");

      // any changes that happen after now cause abort; see below
      gSettings.SyncStart();

      var local = new BookmarkSet();
      var remote = new BookmarkSet();
      var localCopy = new BookmarkSet();

      local.SetDatasource("rdf:bookmarks");
      localCopy.CopyFromDatasourceAS(local, null, false, null, c4);
      return;

    case 404: // not found
      if (Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
          getService(Components.interfaces.nsIPromptService).
          confirm(null, sb("appname.long"), sb("msg.syncFileMissing"))) {
        LogWrite("File not found; CopyLocalToRemote");
        CopyLocalToRemote(false, false, null, true);
      } else {
        ReturnSuccess("msg.cancelled");
      }
      return;

    case 304: // not modified

      CheckLocalDirty(c5);
      return;

    case 401: // unauthorized
      if (Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
          getService(Components.interfaces.nsIPromptService).
          confirm(null, sb("appname.long"), sb("msg.invalidcredentials"))) {
          LogWrite("Opening dialog box");
          ReturnErrorCode(status);
          OpenFoxmarksSettingsDialog();
      } else {
          ReturnErrorCode(status);
      }
      return;

    case 403: // forbidden
      Handle403();
      ReturnErrorCode(status);
      return;

    default: // unhandled error
      ReturnErrorCode(status);  // error
      return;
  }

  function c1(rv) {
    LogWrite("Sync completed; rv=" + rv);

    if (rv < 0) {   // sync failed
      ReturnErrorCode(2); // error (cancel?)
    } else {        // sync succeeded

      if (dirty.remote) {   // update local if necessary

        // ensure that local hasn't changed before writing;
        // abort if necessary
        local.HasDirtyResourcesAS(gSettings.SyncStarted, c2);
        return;
      }
      c3();
    }
  }

  function c2(localIsDirty) {
    if (localIsDirty) {
      ReturnErrorCode(1004);
      return;
    }

    local.CopyFromDatasourceAS(localCopy, null, false, null, c3);
    return;
  }

  function c3() {
    if (dirty.local) {    // update remote if necessary
      StartUpload(remote, etag, gSettings.token ? gSettings.token : attrs.token);
    } else {
      gSettings.SyncComplete(etag);
      ReturnSuccess("msg.synccompleted"); // okay
    }
  }

  function c4() {
    if (remote.CreateFromString(gDownloader.data, attrs) < 0) {
      ReturnErrorCode(1000);
      return;
    }
    
    if (attrs.token.length && gSettings.token.length &&
        attrs.token != gSettings.token) {
      LogWrite("Token mismatch: " + attrs.token + " != " + gSettings.token);
      switch (DiscontinuityPrompt()) {
        case 0: CopyRemoteToLocal(true, etag); return;
        case 1: CopyRemoteToLocal(false, etag); return;
        case 2: ReturnSuccess("msg.cancelled"); return;
      }
    }
    
    LogWrite("Starting sync");
    Sync(localCopy, remote, gSettings.lastSynchDate, dirty, c1);
  }

  function c5(dirty) {
    if (dirty) {
      CopyLocalToRemote(false, true, etag, false);
    } else {
      gSettings.SyncComplete(etag);
      ReturnSuccess("msg.nochanges");
    }
    return;
  }
  
  function DiscontinuityPrompt() {
    var sb = Bundle().GetStringFromName;

    // get a reference to the prompt service component.
    var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
      getService(Components.interfaces.nsIPromptService);

    var flags = ps.BUTTON_TITLE_IS_STRING * ps.BUTTON_POS_0 +
          ps.BUTTON_TITLE_IS_STRING * ps.BUTTON_POS_1 +
          ps.BUTTON_TITLE_CANCEL    * ps.BUTTON_POS_2;
          
    rv = ps.confirmEx(null, sb("disc.title"), sb("disc.body"), flags,
        sb("disc.merge"), sb("disc.download"), null, null, {});
        
    return rv;

  }
  
}

function DownloadCallbackSynch(status, etag)
{
  // check to see whether we'd already synched to this source. If no, then:
  //  if we successfully downloaded a file, ask user whether local or remote should become the master
  //  if the file mentioned is not there, then assume that local is the master
  // if we have already synched to this remote file, then begin the sync process

  if (!gSettings.HaveSynched) {
    FirstSync(status, etag);
  } else {
    NormalSync(status, etag);
  }
}

// Internal functions

function CheckLocalDirty(callback)
{
  // for our purposes, we're not dirty if we've never synced
  if (!gSettings.HaveSynched) {
    callback(false);
  }

  var local = new BookmarkSet();
  local.SetDatasource("rdf:bookmarks");
  local.HasDirtyResourcesAS(gSettings.lastSynchDate, callback);
}

function LocalOrRemote()
{
  var rv;

  if (gPrompt) {

   var sb = Bundle();

    // get a reference to the prompt service component.
    var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
      getService(Components.interfaces.nsIPromptService);

    var flags = ps.BUTTON_TITLE_IS_STRING * ps.BUTTON_POS_0 +
          ps.BUTTON_TITLE_IS_STRING * ps.BUTTON_POS_1 +
          ps.BUTTON_TITLE_CANCEL    * ps.BUTTON_POS_2;

    rv = ps.confirmEx(null, "Initial Synchronization",
      sb.GetStringFromName("msg.initialsync"),
      flags, sb.GetStringFromName("label.local"),
      sb.GetStringFromName("label.remote"), null,
      sb.GetStringFromName("label.merge"), gMerge);
  } else {    // values were provided by caller

    gMerge.value = gDoMerge;
    rv = gRemoteIsMaster ? 1 : 0;

  }

  return rv;
}

function CopyLocalToRemote(doMerge, selectiveDirty, etag, makeNewToken)
{
  var local = new BookmarkSet();
  var localCopy = new BookmarkSet();
  local.SetDatasource("rdf:bookmarks");
  // make a copy of local with everything dirtied
  localCopy.CopyFromDatasourceAS(local, null, true,
    selectiveDirty ? gSettings.lastSynchDate : null, doMerge ? c1 : c2);
  return;

  function c1() {
    // merge case
    SetProgressMessage("progress.merging");
    var remote = new BookmarkSet();

    if (remote.CreateFromString(gDownloader.data, null) < 0) {
      ReturnErrorCode(1000);  // error
      return;
    }

    Merge(localCopy, remote);
    local.CopyFromDatasourceAS(localCopy, null, false, null, c2);
  }

  function c2() {
    // non-merge case
    StartUpload(localCopy, etag, makeNewToken ? null : gSettings.token);
    SetProgressMessage("progress.copyinglocal");
  }
}

function CopyRemoteToLocal(doMerge, etag)
{
  var remote = new BookmarkSet();
  var attrs = {};

  if (remote.CreateFromString(gDownloader.data, attrs) < 0) {
    ReturnErrorCode(1000);  // error
    return;
  }

  SetProgressMessage("progress.loading");

  var local = new BookmarkSet();
  local.SetDatasource("rdf:bookmarks");

  if (doMerge) {
    var localCopy = new BookmarkSet();
    localCopy.CopyFromDatasourceAS(local, null, false, null, c3);
    return; // let callback handle the rest
  }

  // non-merge case
  gSettings.SyncStart();
  local.CopyFromDatasourceAS(remote, null, false, null, c1);
  return;

  function c1() {
    gSettings.token = attrs.token;
    gSettings.SyncComplete(etag);
    ReturnSuccess("msg.remotefilecopied");
  }

  function c2() {
    StartUpload(remote, etag, attrs.token);
  }

  function c3() {
    SetProgressMessage("progress.merging");
    Merge(remote, localCopy);
    local.CopyFromDatasourceAS(remote, null, false, null, c2);
  }
}

function LogStart(op, dest)
{
  LogWrite("------ Foxmarks/" + FoxmarksVersion() + " starting " + op + " with " + dest + " ------");
}

///////////////////////////////////////////////////////////////////////////////
//
// nsFoxmarksService
//
function nsFoxmarksService()
{
  LoadFiles();
}

nsFoxmarksService.prototype = {
  /////////////////////////////////////////////////////////////////////////////
  // nsIFoxmarksService

  timer: null,

  synchronize: function (observer)
  {
    // return if we're currently processing another request
    if (IsBusy(observer)) {
      return;
    }

    LogStart("sync", gSettings.URL);

    gPrompt = true;
    SetProgressMessage(gSettings.HaveSynched ? "progress.downloading" : "progress.searching");
    gDownloader.start(gSettings.downloadURL, gSettings.etag, DownloadCallbackSynch);
  },

  synchronizeInitial: function (observer, remoteIsMaster, doMerge)
  {
    // return if we're currently processing another request
    if (IsBusy(observer)) {
      return;
    }

    LogStart("initial sync", gSettings.URL);

    gPrompt = false;
    gRemoteIsMaster = remoteIsMaster;
    gDoMerge = doMerge;

    SetProgressMessage(gSettings.HaveSynched ? "progress.downloading" : "progress.searching");
    gDownloader.start(gSettings.downloadURL, gSettings.etag, DownloadCallbackSynch);
  },



  upload: function (observer)
  {
    if (IsBusy(observer)) {
      return;
    }

    LogStart("upload", gSettings.URL);

    CopyLocalToRemote(false, false, null, true);
  },

  download: function (observer)
  {
    // return if we're currently processing another request
    if (IsBusy(observer)) {
      return;
    }

    LogStart("download", gSettings.URL);

    SetProgressMessage("progress.downloading");
    gDownloader.start(gSettings.downloadURL, "", DownloadCallbackOverwrite);
  },

  verifyAccount: function (observer, username, password)
  {
    // return if we're currently processing another request
    if (IsBusy(observer)) {
      return;
    }

    // store these off to be used later by callback
    gAccount.username = username;
    gAccount.password = password;

    LogStart("Verifying account", username + "@" + gSettings.wizardHostName);
    SetProgressMessage("progress.verifying");

//  Drat! We can't use the API below unless we're an administrator
//    gDownloader.start("http://" + username + ":" + password +
//      "@sync.foxcloud.com/api/user/" + username, "", DownloadCallbackVerify);
//  ... so for now, just do this:
    gDownloader.start("http://" + username + ":" + password +
      "@" + gSettings.wizardHostName + "/home/" + username + "/", "", DownloadCallbackVerify);

  },

  createAccount: function (observer, username, password, firstName, lastName, email)
  {
    if (IsBusy(observer)) {
      return;
    }

    LogStart("createAccount", username + "@" + gSettings.wizardHostName);

    // store these off to be used later by callback
    gAccount.username = username;
    gAccount.password = password;
    gAccount.firstName = firstName;
    gAccount.lastName = lastName;
    gAccount.email = email;

    var body = '\
<?xml version="1.0" encoding="utf-8" ?>\n\
<user xmlns="http://osafoundation.org/cosmo">\n\
<username>{username}</username>\n\
<password>{password}</password>\n\
<firstName>{firstName}</firstName>\n\
<lastName>{lastName}</lastName>\n\
<email>{email}</email>\n\
</user>\n';

    body = body.replace('{username}', username);
    body = body.replace('{password}', password);
    body = body.replace('{firstName}', firstName);
    body = body.replace('{lastName}', lastName);
    body = body.replace('{email}', email);

    SetProgressMessage("progress.creating");
    gUploader.start("http://" + gSettings.wizardHostName + "/api/signup", null, body, CreateAccountCallback);
  },

  verifyFile: function (observer)
  {
    if (IsBusy(observer)) {
      return;
    }

   LogStart("verify", gSettings.URL);

   SetProgressMessage("progress.searching");
   gDownloader.start(gSettings.downloadURL, "", VerifyFileCallback);
  },

  cancel: function()
  {
    gDownloader.cancel();
    gUploader.cancel();
  },

  logWrite: function (msg)
  {
    if (!logStream)
       logFileOpen();

    var d = new Date();
    var year = d.getFullYear();
    var month = d.getMonth() + 1; if (month < 10) month = "0" + month;
    var day = d.getDate(); if (day < 10) day = "0" + day;
    var hour = d.getHours(); if (hour < 10) hour = "0" + hour;
    var minute = d.getMinutes(); if (minute < 10) minute = "0" + minute;
    var sec = d.getSeconds(); if (sec < 10) sec = "0" + sec;

    // format is [YYYY-MM-DD HH:MM:SS] msg\n
    var string = "[" + year + "-" + month + "-" + day + " " + hour + ":" + minute + ":" + sec + "] " + msg + "\n";

    logStream.write(string, string.length)
  },


  /////////////////////////////////////////////////////////////////////////////
  //
  // nsIObserver

   observe: function(subject, topic, data)
   {
      var timerCallback = {
        parent: this,

        notify: function(timer) {
          var now = new Date().getTime();
          
          if (gSettings.daysSinceLastUpdateNag > 28 && FoxmarksUpdateAvailable()) {
            gSettings.lastNagDate = gSettings.NowAsGMT;
            UpdateNag();
          }

          if (!gObserver && (!gFailureCount || now > gBackoffUntil) && gSettings.synchOnTimer && gSettings.HaveSynched) {
            if (gSettings.minutesSinceLastSync > gSettings.autoSynchFreq) {
              FireSync();
            } else {
              CheckLocalDirty((function(dirty) { if (dirty) FireSync() } ));
            }
          }

          function FireSync() {
            // find a browser window to show our status
            var topWindow = Components.classes['@mozilla.org/appshell/window-mediator;1'].
                getService(Components.interfaces.nsIWindowMediator).
                getMostRecentWindow("navigator:browser");

            // okay, this is weird
            var foxmarks = Components.classes["@foxcloud.com/extensions/foxmarks;1"].
              getService(Components.interfaces.nsIFoxmarksService);

            if (topWindow && topWindow.gFoxmarksObserver)
              foxmarks.synchronize(topWindow.gFoxmarksObserver);
          }
        }
      }

      this.timer = Components.classes["@mozilla.org/timer;1"].
        createInstance(Components.interfaces.nsITimer);

      this.timer.initWithCallback(timerCallback, 1000*60,
        Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);

  },

  /////////////////////////////////////////////////////////////////////////////
  // nsIClassInfo
  getInterfaces: function (aCount)
  {
    var interfaces = [Components.interfaces.nsIFoxmarksService,
                      Components.interfaces.nsIObserver];
    aCount.value = interfaces.length;
    return interfaces;
  },

  getHelperForLanguage: function (aLanguage)
  {
    return null;
  },

  get contractID()
  {
    return "@foxcloud.com/extensions/foxmarks;1";
  },

  get classDescription()
  {
    return "Foxmarks Service";
  },

  get classID()
  {
    return Components.ID("{49ace257-6111-48b2-a988-f9eb38b0fa58}");
  },

  get implementationLanguage()
  {
    return Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT;
  },

  get flags()
  {
    return Components.interfaces.nsIClassInfo.SINGLETON;
  },

  /////////////////////////////////////////////////////////////////////////////
  // nsISupports
  QueryInterface: function (aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFoxmarksService) &&
        !aIID.equals(Components.interfaces.nsISupports) &&
        !aIID.equals(Components.interfaces.nsIObserver))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

function FoxmarksObserver()
{
}

FoxmarksObserver.prototype = {
  /////////////////////////////////////////////////////////////////////////////
  // nsIClassInfo
  getInterfaces: function (aCount)
  {
    var interfaces = [Components.interfaces.nsIFoxmarksObserver];
    aCount.value = interfaces.length;
    return interfaces;
  },

  getHelperForLanguage: function (aLanguage)
  {
    return null;
  },

  get contractID()
  {
    return "@foxcloud.com/extensions/foxmarks-observer;1";
  },

  get classDescription()
  {
    return "FoxmarksObserver";
  },

  get classID()
  {
    return Components.ID("{e305cb5d-7878-4f4f-b104-28bdd056eb90}");
  },

  get implementationLanguage()
  {
    return Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT;
  },

  get flags()
  {
    return Components.interfaces.nsIClassInfo.SINGLETON;
  },

  /////////////////////////////////////////////////////////////////////////////
  // nsISupports
  QueryInterface: function (aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFoxmarksObserver))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};


var gModule = {
  _firstTime: true,

  registerSelf: function (aComponentManager, aFileSpec, aLocation, aType)
  {
    if (this._firstTime) {
      this._firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }


    aComponentManager = aComponentManager.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    for (var key in this._objects) {
      var obj = this._objects[key];
      aComponentManager.registerFactoryLocation(obj.CID, obj.className, obj.contractID,
                                                aFileSpec, aLocation, aType);
    }

    // Make the Foxmarks Service a startup observer
    var cm = Components.classes["@mozilla.org/categorymanager;1"].
        getService(Components.interfaces.nsICategoryManager);
    cm.addCategoryEntry("app-startup", this._objects.service.className,
        "service," + this._objects.service.contractID, true, true, null);
  },


  getClassObject: function (aComponentManager, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    for (var key in this._objects) {
      if (aCID.equals(this._objects[key].CID))
        return this._objects[key].factory;
    }

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  _makeFactory: #1= function(ctor) {
     return {
              createInstance: function (outer, iid) {
                if (outer != null)
                  throw Components.results.NS_ERROR_NO_AGGREGATION;
                return (new ctor()).QueryInterface(iid);
              }
            };
  },

  _objects: {
    observer:{ CID         : FoxmarksObserver.prototype.classID,
               contractID  : FoxmarksObserver.prototype.contractID,
               className   : FoxmarksObserver.prototype.classDescription,
               factory     : #1#(FoxmarksObserver)
             },

    service: { CID        : nsFoxmarksService.prototype.classID,
               contractID : nsFoxmarksService.prototype.contractID,
               className  : nsFoxmarksService.prototype.classDescription,
               factory    : #1#(nsFoxmarksService)
             }
   },

  canUnload: function (aComponentManager)
  {
    return true;
  }
};

function NSGetModule(compMgr, fileSpec)
{
  return gModule;
}
